@@ -2,7 +2,8 @@ __author__ = """Jean-Paul Ladage <j.ladage@zestsoftware.nl>"""
 __docformat__ = 'plaintext'
 
 from AccessControl import ClassSecurityInfo
-from Acquisition import aq_chain
+from Acquisition import aq_chain, aq_inner
+from DateTime import DateTime
 from Products.Archetypes.utils import IntDisplayList
 from Products.Archetypes.interfaces import IBaseContent
 from Products.Archetypes.atapi import BaseContent
@@ -29,7 +30,6 @@ from plonehrm.contracts import config
 from plonehrm.contracts.interfaces import IContract
 from plonehrm.contracts import ContractsMessageFactory as _
 
-
 schema = Schema((
     StringField(name='template',
         required=1,
@@ -121,6 +121,18 @@ schema = Schema((
             label=_(u'contract_label_hours', default=u'Number of hours'),
         )
     ),
+    IntegerField(
+        name='daysPerWeek',
+        read_permission='plonehrm: View hrm content',
+        write_permission='plonehrm: Modify contract',
+        default_method='default_days_per_week',
+        validators = ('maxDaysPerWeek', ),
+        widget=IntegerWidget(
+            condition='not:object/template_chosen',
+            label=_(u'contract_label_days_per_week',
+                    default=u'Number of workdays per week'),
+        )
+    ),
     TextField(name='text',
         required=False,
         seachable=False,
@@ -219,7 +231,7 @@ class Contract(BaseContent):
         parent is not an Employee.  But we can traverse up the
         acquisition chain to find it.
         """
-        for parent in aq_chain(self):
+        for parent in aq_chain(aq_inner(self)):
             if IEmployee.providedBy(parent):
                 return parent
 
@@ -270,4 +282,40 @@ class Contract(BaseContent):
             return ''
         return base.getEmploymentType()
 
+    def default_days_per_week(self):
+        """Get the days per week of the parent Employee.
+        """
+        base = self.base_contract()
+        if base is None or base == self:
+            return '0'
+        return base.getDaysPerWeek()
+
+    def expiry_date(self):
+        """Expiry date of this contract.
+
+        Basically start date + duration.  But e.g. a contract of one
+        month starting at 31 January should end at 28 (or 29) February.
+
+        Will return None if start date or duration is not known.
+        """
+        start = self.getStartdate()
+        duration = self.getDuration()
+        if start is None or duration is None:
+            return None
+        year, month, day = start.year(), start.month(), start.day()
+        year = year + (month + duration - 1) / 12
+        month = 1 + (month + duration - 1) % 12
+        first_of_month = DateTime(year, month, 1)
+        date = first_of_month + day - 1
+        # 31 January + 31 days is 3 March, so count backwards in that
+        # case.  Watch out for infinite loops here...
+        safety_count = 5
+        while date.month() != month and safety_count > 0:
+            safety_count -= 1
+            date -= 1
+        if not safety_count:
+            return None
+        return date
+
+
 registerType(Contract, config.PROJECTNAME)
